/*------------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        29.05.2017
 Description:    DMX_Switch
------------------------------------------------------------------------------*/

#include <avr/interrupt.h>
#include <avr/io.h>
#include <stdio.h>

#define F_CPU 12000000

#define LED_OUT		DDRD |= (1<<PD5);
#define LED_TOGGLE	PORTD ^= (1<<PD5);

#define DMX_BAUD 250000
#define DMX_LOST_TIMEOUT 8000
	
volatile unsigned char dmx_buffer[513];
volatile unsigned int dmx_lost = DMX_LOST_TIMEOUT;

volatile unsigned int dmx_adresse = 0;

//############################################################################
//Empfangsroutine fr DMX
ISR (USART_RX_vect){
	static unsigned int dmx_channel_rx_count = 0;
	static unsigned char dmx_valid = 0;
	unsigned char tmp = 0;
	
	tmp =  UDR0;
	
	if(UCSR0A&(1<<FE0)){
		if(dmx_channel_rx_count > 1){
			dmx_lost = 0;
		}
		dmx_channel_rx_count = 0;	
		dmx_buffer[0] = tmp;
		if(dmx_buffer[0] == 0){
			dmx_valid = 1;
			dmx_channel_rx_count++;
		}else{
			dmx_valid = 0;
		}
		return;
	}
	
	if(dmx_valid){
		dmx_buffer[dmx_channel_rx_count] = tmp;	
		if(dmx_channel_rx_count < 513)
		{
			dmx_channel_rx_count++;
		}
		return;
	}
}

//############################################################################
//Hier wird die Zeit gezhlt (Tick 100s)
ISR (TIMER2_COMPA_vect)
{
	if(dmx_lost<DMX_LOST_TIMEOUT){
		dmx_lost++;
	}
}

//############################################################################
//Hauptprogramm
int main (void) 
{  
	unsigned int dmx_adresse_tmp = 0;
	unsigned int live_counter = 0;

	LED_OUT;

	//Relai Output Aus
	DDRB |= (1<<PB0);
	PORTB &= ~(1<<PB0);

	//Init usart DMX-BUS
	UBRR0   = (F_CPU / (DMX_BAUD * 16L) - 1);
	UCSR0B|=(1 << RXEN0 | 1<< RXCIE0);
	UCSR0C|=(1<<USBS0); //USBS0 2 Stop bits	
	sei();//Globale Interrupts Enable
	
	//Switch 75176 to INPUT RXD
	DDRD |= (1<<PD2);
	PORTD &= ~(1<<PD2);
	
	//Timer
	TCCR2A |= (1<<WGM21);
	TCCR2B |= (1<<CS21|1<<CS20);
	TIMSK2 |= (1<<OCIE2A);
	OCR2A = (F_CPU/32/2/50/240) - 1;
	
	//Pullup for DIP Switch
	PORTD |= (1<<PD3)|(1<<PD4)|(1<<PD7);
	PORTC |= (1<<PC5)|(1<<PC4)|(1<<PC3)|(1<<PC2)|(1<<PC1)|(1<<PC0);
	PORTB |= (1<<PB1);
	
	//Endlosschleife
	while(1){
		//Read DMX Address from Switch
		dmx_adresse_tmp = 0;
		if(!(PIND&(1<<PD4))) dmx_adresse_tmp |= 0x01;
 		if(!(PIND&(1<<PD3))) dmx_adresse_tmp |= 0x02;
		if(!(PINC&(1<<PC5))) dmx_adresse_tmp |= 0x04;
		if(!(PINC&(1<<PC4))) dmx_adresse_tmp |= 0x08;
		if(!(PINC&(1<<PC3))) dmx_adresse_tmp |= 0x10;
		if(!(PINC&(1<<PC2))) dmx_adresse_tmp |= 0x20;
		if(!(PINC&(1<<PC1))) dmx_adresse_tmp |= 0x40;
		if(!(PINC&(1<<PC0))) dmx_adresse_tmp |= 0x80;
		if(!(PINB&(1<<PB1))) dmx_adresse_tmp |= 0x0100;
		
		if(dmx_adresse_tmp == 0) dmx_adresse_tmp = 1;  
		if(dmx_adresse_tmp > 512) dmx_adresse_tmp = 512;
		if(dmx_adresse != dmx_adresse_tmp) dmx_adresse =  dmx_adresse_tmp;	
		
		//new calculation of val

		if(dmx_buffer[dmx_adresse]>128){
			//Relai an
			PORTB |= (1<<PB0);
		}else{
			//Relai aus
			PORTB &= ~(1<<PB0);
		}
		
		if(dmx_lost==DMX_LOST_TIMEOUT){
			//Relai aus
			PORTB &= ~(1<<PB0);
		}
		
		if((live_counter++) > 10000) {
			LED_TOGGLE;
			live_counter = 0;
		}	
	}
}